/*      -*- linux-c -*-
 *
 * Copyright (c) 2003 by Intel Corp.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  This
 * file and program are licensed under a BSD style license.  See
 * the Copying file included with the OpenHPI distribution for
 * full licensing terms.
 *
 * Authors:
 *     Louis Zhuang <louis.zhuang@linux.intel.com>
 */

#include "ipmi.h"
#include <netdb.h>

static void get_sel_count(ipmi_domain_t *domain, void *cb_data)
{
        ipmi_domain_sel_count(domain, (int *)cb_data);
}

void ohoi_get_sel_count(ipmi_domain_id_t domain_id, int *count)
{
	int rv;
	
        rv = ipmi_domain_pointer_cb(domain_id, get_sel_count, count); 
	if (rv<0)
		dbg("Unable to convert domain id to a pointer");
}

struct get_sel_time_cb_data {
	SaHpiTimeT	time;
	int		flag;
};

static void get_sel_time_cb(ipmi_mc_t	*mc,
		int           		err,
		unsigned long 		time,
		void          		*cb_data)
{
	struct get_sel_time_cb_data *data = cb_data;
	data->flag = 1;
	data->time = time;
}

static void get_sel_time(ipmi_mc_t *mc, void *cb_data)
{
	SaHpiTimeT *time = cb_data;
	struct get_sel_time_cb_data data;
	struct timeval tv1, tv2;
	int rv;

	memset(&data, 0, sizeof(data));
	ipmi_mc_get_current_sel_time(mc, get_sel_time_cb, &data);
	
	gettimeofday(&tv1, NULL);
	tv1.tv_sec += 10;
	while (1) {
		struct timeval tv3;

		if (data.flag)
			goto out;
		memset(&tv3, 0, sizeof(tv3));	
		gettimeofday(&tv2, NULL);
		if (tv2.tv_sec>tv1.tv_sec) break;
		rv = sel_select(ui_sel, NULL, 0, NULL, &tv3);
	}
		
out:
	*time = (SaHpiTimeT)data.time*1000000000;
}

void ohoi_get_sel_time(ipmi_domain_id_t domain_id, SaHpiTimeT *time)
{
	int rv;	
	ipmi_mcid_t   mc_id;

	mc_id.channel 	= 0x00;
	mc_id.mc_num  	= 0x20;
	mc_id.domain_id = domain_id;
	rv = _ipmi_mc_pointer_noseq_cb(mc_id, get_sel_time, time); 
	if (rv<0)
		dbg("Unable to convert domain id to a pointer");
}

static void get_sel_overflow(ipmi_mc_t *mc, void *cb_data)
{
	char *overflow = cb_data;
	*overflow =  _ipmi_mc_sel_get_overflow(mc);
}

void ohoi_get_sel_overflow(ipmi_domain_id_t domain_id, char *overflow)
{
	int rv;	
	ipmi_mcid_t   mc_id;

	mc_id.channel 	= 0x00;
	mc_id.mc_num  	= 0x20;
	mc_id.domain_id = domain_id;
	rv = _ipmi_mc_pointer_noseq_cb(mc_id, get_sel_overflow, overflow); 
	if (rv<0)
		dbg("Unable to convert domain id to a pointer");
	
}

static void get_sel_support_del(ipmi_mc_t *mc, void *cb_data)
{
	char *support_del = cb_data;

	*support_del = _ipmi_mc_sel_get_supports_delete_sel(mc);
}

void ohoi_get_sel_support_del(ipmi_domain_id_t domain_id, char *support_del)
{
	int rv;	
	ipmi_mcid_t   mc_id;

	mc_id.channel 	= 0x00;
	mc_id.mc_num  	= 0x20;
	mc_id.domain_id = domain_id;
	rv = _ipmi_mc_pointer_noseq_cb(mc_id, get_sel_support_del, support_del); 
	if (rv<0)
		dbg("Unable to convert domain id to a pointer");
}

struct set_sel_time_cb_data {
	int		flag;
};

//static void set_sel_time_done(ipmi_mc_t	*mc,
		//int           		err,
		//void          		*cb_data)
//{
	//struct get_sel_time_cb_data *data = cb_data;
	//data->flag = 1;
//}


void ohoi_set_sel_time(ipmi_mc_t *mc, const struct timeval *time)
{
	struct set_sel_time_cb_data data;
	struct timeval tv1, tv2;
	int rv;
	
	memset(&data, 0, sizeof(data));
	//ipmi_mc_set_current_sel_time(mc, time, set_sel_time_done, &data);
	
	gettimeofday(&tv1, NULL);
	tv1.tv_sec += 10;
	while (1) {
		struct timeval tv3;

		if (data.flag)
			goto out;
		memset(&tv3, 0, sizeof(tv3));	
		gettimeofday(&tv2, NULL);
		if (tv2.tv_sec>tv1.tv_sec) break;
		rv = sel_select(ui_sel, NULL, 0, NULL, &tv3);
	}
		
	dbg("Unable to get sensor thresholds: Timeout!\n");
out:
	return;
}
